@@ -50,6 +50,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <sysexits.h>
 #include <dirent.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -58,7 +59,6 @@
 #include <libxml/tree.h>
 
 #include "annotate.h" /* for strlist functionality */
-#include "exitcodes.h"
 #include "global.h"
 #include "hash.h"
 #include "map.h"
@@ -88,7 +88,7 @@ int main(int argc, char **argv)
 {
     int opt, r = 0;
     char *alt_config = NULL, *pub = NULL, *ver = NULL, *winfile = NULL;
-    char prefix[2048];
+    const char *zoneinfo_dir = NULL;
     enum { REBUILD, WINZONES, NONE } op = NONE;
 
     while ((opt = getopt(argc, argv, "C:r:vw:")) != EOF) {
@@ -130,14 +130,19 @@ int main(int argc, char **argv)
     signals_set_shutdown(&shut_down);
     signals_add_handlers(0);
 
-    snprintf(prefix, sizeof(prefix), "%s%s", config_dir, FNAME_ZONEINFODIR);
+    zoneinfo_dir = config_getstring(IMAPOPT_ZONEINFO_DIR);
+    if (!zoneinfo_dir) {
+        fprintf(stderr, "zoneinfo_dir must be set for tzdist service\n");
+        cyrus_done();
+        return EX_CONFIG;
+    }
 
     switch (op) {
     case REBUILD: {
         struct hash_table tzentries;
         struct zoneinfo *info;
         struct txn *tid = NULL;
-        char buf[1024];
+        char buf[3000];
         FILE *fp;
 
         construct_hash_table(&tzentries, 500, 1);
@@ -150,7 +155,7 @@ int main(int argc, char **argv)
         hash_insert(INFO_TZID, info, &tzentries);
 
         /* Add LEAP record (last updated and hash) */
-        snprintf(buf, sizeof(buf), "%s%s", prefix, FNAME_LEAPSECFILE);
+        snprintf(buf, sizeof(buf), "%s%s", zoneinfo_dir, FNAME_LEAPSECFILE);
         if (verbose) printf("Processing leap seconds file %s\n", buf);
         if (!(fp = fopen(buf, "r"))) {
             fprintf(stderr, "Could not open leap seconds file %s\n", buf);
@@ -187,7 +192,7 @@ int main(int argc, char **argv)
         }
 
         /* Add ZONE/LINK records */
-        do_zonedir(prefix, &tzentries, info);
+        do_zonedir(zoneinfo_dir, &tzentries, info);
 
         zoneinfo_open(NULL);
 
@@ -243,8 +248,8 @@ int main(int argc, char **argv)
             goto done;
         }
 
-        if (chdir(prefix)) {
-            fprintf(stderr, "chdir(%s) failed\n", prefix);
+        if (chdir(zoneinfo_dir)) {
+            fprintf(stderr, "chdir(%s) failed\n", zoneinfo_dir);
             goto done;
         }
 
@@ -324,7 +329,7 @@ void usage(void)
     fprintf(stderr,
             "usage: zoneinfo_reconstruct [-C <alt_config>] [-v]"
             " -r <publisher>:<version>\n");
-    exit(EC_USAGE);
+    exit(EX_USAGE);
 }
 
 
@@ -371,7 +376,7 @@ void do_zonedir(const char *dir, struct hash_table *tzentries,
 
             /* Isolate alias in path */
             path[plen-4] = '\0';  /* Trim ".ics" */
-            alias = path + strlen(config_dir) + strlen("zoneinfo") + 2;
+            alias = path + strlen(dir) + 1;
 
             if (verbose) printf("\tLINK: %s -> %s\n", alias, tzid);
 
@@ -398,6 +403,7 @@ void do_zonedir(const char *dir, struct hash_table *tzentries,
             size_t len = 0;
             icalcomponent *ical, *comp;
             icalproperty *prop;
+            char *alias = NULL;
 
             /* Parse the iCalendar file for important properties */
             if ((fd = open(path, O_RDONLY)) == -1) continue;
@@ -414,7 +420,15 @@ void do_zonedir(const char *dir, struct hash_table *tzentries,
             prop = icalcomponent_get_first_property(comp, ICAL_TZID_PROPERTY);
             tzid = (char *) icalproperty_get_value_as_string(prop);
 
-            if (verbose) printf("\tZONE: %s\n", tzid);
+            prop = icalcomponent_get_first_property(comp,
+                                                    ICAL_TZIDALIASOF_PROPERTY);
+            if (prop) {
+                alias = tzid;
+                tzid = (char *) icalproperty_get_value_as_string(prop);
+
+                if (verbose) printf("\tLINK: %s -> %s\n", alias, tzid);
+            }
+            else if (verbose) printf("\tZONE: %s\n", tzid);
 
             /* Create/update hash entry for tzid */
             if (!(zi = hash_lookup(tzid, tzentries))) {
@@ -430,6 +444,19 @@ void do_zonedir(const char *dir, struct hash_table *tzentries,
 
             /* Check overall lastmod */
             if (zi->dtstamp > info->dtstamp) info->dtstamp = zi->dtstamp;
+
+            if (alias) {
+                /* Add alias to the list for this tzid */
+                appendstrlist(&zi->data, alias);
+
+                /* Create hash entry for alias */
+                if (!(zi = hash_lookup(alias, tzentries))) {
+                    zi = xzmalloc(sizeof(struct zoneinfo));
+                    hash_insert(alias, zi, tzentries);
+                }
+                zi->type = ZI_LINK;
+                appendstrlist(&zi->data, tzid);
+            }
         }
         else {
             fprintf(stderr, "unknown path type %s\n", path);
