@@ -62,11 +62,13 @@
 #include "global.h"
 #include "xmalloc.h"
 #include "mailbox.h"
-#include "imap/imap_err.h"
 #include "seen.h"
 #include "sync_log.h"
 #include "imparse.h"
 
+/* generated headers are not necessarily in current directory */
+#include "imap/imap_err.h"
+
 #define FNAME_SEENSUFFIX ".seen" /* per user seen state extension */
 #define FNAME_SEEN "/cyrus.seen" /* for legacy seen state */
 
@@ -76,9 +78,9 @@ enum {
 };
 
 struct seen {
-    char *user;			/* what user is this for? */
+    char *user;                 /* what user is this for? */
     struct db *db;
-    struct txn *tid;		/* outstanding txn, if any */
+    struct txn *tid;            /* outstanding txn, if any */
 };
 
 #define DB (config_seenstate_db)
@@ -86,30 +88,30 @@ struct seen {
 HIDDEN char *seen_getpath(const char *userid)
 {
     char *fname = xmalloc(strlen(config_dir) + sizeof(FNAME_DOMAINDIR) +
-			  sizeof(FNAME_USERDIR) + strlen(userid) +
-			  sizeof(FNAME_SEENSUFFIX) + 10);
+                          sizeof(FNAME_USERDIR) + strlen(userid) +
+                          sizeof(FNAME_SEENSUFFIX) + 10);
     char c, *domain;
 
     if (config_virtdomains && (domain = strchr(userid, '@'))) {
-	char d = (char) dir_hash_c(domain+1, config_fulldirhash);
-	*domain = '\0';  /* split user@domain */
-	c = (char) dir_hash_c(userid, config_fulldirhash);
-	sprintf(fname, "%s%s%c/%s%s%c/%s%s", config_dir, FNAME_DOMAINDIR, d,
-		domain+1, FNAME_USERDIR, c, userid, FNAME_SEENSUFFIX);
-	*domain = '@';  /* reassemble user@domain */
+        char d = (char) dir_hash_c(domain+1, config_fulldirhash);
+        *domain = '\0';  /* split user@domain */
+        c = (char) dir_hash_c(userid, config_fulldirhash);
+        sprintf(fname, "%s%s%c/%s%s%c/%s%s", config_dir, FNAME_DOMAINDIR, d,
+                domain+1, FNAME_USERDIR, c, userid, FNAME_SEENSUFFIX);
+        *domain = '@';  /* reassemble user@domain */
     }
     else {
-	c = (char) dir_hash_c(userid, config_fulldirhash);
-	sprintf(fname, "%s%s%c/%s%s", config_dir, FNAME_USERDIR, c, userid,
-		FNAME_SEENSUFFIX);
+        c = (char) dir_hash_c(userid, config_fulldirhash);
+        sprintf(fname, "%s%s%c/%s%s", config_dir, FNAME_USERDIR, c, userid,
+                FNAME_SEENSUFFIX);
     }
 
     return fname;
 }
 
 EXPORTED int seen_open(const char *user,
-	      int flags,
-	      struct seen **seendbptr)
+              int flags,
+              struct seen **seendbptr)
 {
     struct seen *seendb = NULL;
     char *fname = NULL;
@@ -123,7 +125,7 @@ EXPORTED int seen_open(const char *user,
     seendb = (struct seen *) xmalloc(sizeof(struct seen));
 
     if (SEEN_DEBUG) {
-	syslog(LOG_DEBUG, "seen_db: seen_open(%s)", user);
+        syslog(LOG_DEBUG, "seen_db: seen_open(%s)", user);
     }
 
     /* open the seendb corresponding to user */
@@ -131,15 +133,15 @@ EXPORTED int seen_open(const char *user,
     if (flags & SEEN_CREATE) cyrus_mkdir(fname, 0755);
     r = cyrusdb_open(DB, fname, dbflags | CYRUSDB_CONVERT, &seendb->db);
     if (r) {
-	if (!(flags & SEEN_SILENT)) {
-	    int level = (flags & SEEN_CREATE) ? LOG_ERR : LOG_DEBUG;
-	    syslog(level, "DBERROR: opening %s: %s", fname, 
-		   cyrusdb_strerror(r));
-	}
-	r = IMAP_IOERROR;
-	free(seendb);
-	free(fname);
-	return r;
+        if (!(flags & SEEN_SILENT)) {
+            int level = (flags & SEEN_CREATE) ? LOG_ERR : LOG_DEBUG;
+            syslog(level, "DBERROR: opening %s: %s", fname,
+                   cyrusdb_strerror(r));
+        }
+        r = IMAP_IOERROR;
+        free(seendb);
+        free(fname);
+        return r;
     }
     syslog(LOG_DEBUG, "seen_db: user %s opened %s", user, fname);
     free(fname);
@@ -185,10 +187,10 @@ static void parse_data(const char *data, int datalen, struct seendata *sd)
 }
 
 static int foreach_proc(void *rock,
-		 const char *key,
-		 size_t keylen,
-		 const char *data,
-		 size_t datalen)
+                 const char *key,
+                 size_t keylen,
+                 const char *data,
+                 size_t datalen)
 {
     struct seendata sd = SEENDATA_INITIALIZER;
     struct seendata_rock *sr = (struct seendata_rock *)rock;
@@ -214,7 +216,7 @@ EXPORTED int seen_foreach(struct seen *seendb, seenproc_t *f, void *rock)
 }
 
 static int seen_readit(struct seen *seendb, const char *uniqueid,
-		       struct seendata *sd, int rw)
+                       struct seendata *sd, int rw)
 {
     int r;
     const char *data;
@@ -222,38 +224,38 @@ static int seen_readit(struct seen *seendb, const char *uniqueid,
 
     assert(seendb && uniqueid);
     if (rw || seendb->tid) {
-	r = cyrusdb_fetchlock(seendb->db, uniqueid, strlen(uniqueid),
-			  &data, &datalen, &seendb->tid);
+        r = cyrusdb_fetchlock(seendb->db, uniqueid, strlen(uniqueid),
+                          &data, &datalen, &seendb->tid);
     } else {
-	r = cyrusdb_fetch(seendb->db, uniqueid, strlen(uniqueid),
-		      &data, &datalen, NULL);
+        r = cyrusdb_fetch(seendb->db, uniqueid, strlen(uniqueid),
+                      &data, &datalen, NULL);
     }
     switch (r) {
     case 0:
-	break;
+        break;
     case CYRUSDB_AGAIN:
-	syslog(LOG_DEBUG, "deadlock in seen database for '%s/%s'",
-	       seendb->user, uniqueid);
-	return IMAP_AGAIN;
-	break;
+        syslog(LOG_DEBUG, "deadlock in seen database for '%s/%s'",
+               seendb->user, uniqueid);
+        return IMAP_AGAIN;
+        break;
     case CYRUSDB_NOTFOUND:
-	memset(sd, 0, sizeof(struct seendata));
-	sd->seenuids = xstrdup("");
-	return 0;
-	break;
+        memset(sd, 0, sizeof(struct seendata));
+        sd->seenuids = xstrdup("");
+        return 0;
+        break;
     default:
-	syslog(LOG_ERR, "DBERROR: error fetching txn %s",
-	       cyrusdb_strerror(r));
-	return IMAP_IOERROR;
-	break;
+        syslog(LOG_ERR, "DBERROR: error fetching txn %s",
+               cyrusdb_strerror(r));
+        return IMAP_IOERROR;
+        break;
     }
 
     parse_data(data, datalen, sd);
     if (sd->seenuids[0] && !imparse_issequence(sd->seenuids)) {
-	syslog(LOG_ERR, "DBERROR: invalid sequence <%s> for %s %s - nuking",
-	       sd->seenuids, seendb->user, uniqueid);
-	free(sd->seenuids);
-	sd->seenuids = xstrdup("");
+        syslog(LOG_ERR, "DBERROR: invalid sequence <%s> for %s %s - nuking",
+               sd->seenuids, seendb->user, uniqueid);
+        free(sd->seenuids);
+        sd->seenuids = xstrdup("");
     }
 
     return 0;
@@ -262,18 +264,18 @@ static int seen_readit(struct seen *seendb, const char *uniqueid,
 EXPORTED int seen_read(struct seen *seendb, const char *uniqueid, struct seendata *sd)
 {
     if (SEEN_DEBUG) {
-	syslog(LOG_DEBUG, "seen_db: seen_read %s (%s)", 
-	       seendb->user, uniqueid);
+        syslog(LOG_DEBUG, "seen_db: seen_read %s (%s)",
+               seendb->user, uniqueid);
     }
 
     return seen_readit(seendb, uniqueid, sd, 0);
 }
 
-HIDDEN int seen_lockread(struct seen *seendb, const char *uniqueid, struct seendata *sd)
+EXPORTED int seen_lockread(struct seen *seendb, const char *uniqueid, struct seendata *sd)
 {
     if (SEEN_DEBUG) {
-	syslog(LOG_DEBUG, "seen_db: seen_lockread %s (%s)", 
-	       seendb->user, uniqueid);
+        syslog(LOG_DEBUG, "seen_db: seen_lockread %s (%s)",
+               seendb->user, uniqueid);
     }
 
     return seen_readit(seendb, uniqueid, sd, 1);
@@ -289,28 +291,28 @@ EXPORTED int seen_write(struct seen *seendb, const char *uniqueid, struct seenda
     assert(seendb && uniqueid);
 
     if (SEEN_DEBUG) {
-	syslog(LOG_DEBUG, "seen_db: seen_write %s (%s)", 
-	       seendb->user, uniqueid);
+        syslog(LOG_DEBUG, "seen_db: seen_write %s (%s)",
+               seendb->user, uniqueid);
     }
 
-    snprintf(data, sz, "%d %lu %u %lu %s", SEEN_VERSION, 
-	    sd->lastread, sd->lastuid, 
-	    sd->lastchange, sd->seenuids);
+    snprintf(data, sz, "%d %lu %u %lu %s", SEEN_VERSION,
+            sd->lastread, sd->lastuid,
+            sd->lastchange, sd->seenuids);
     datalen = strlen(data);
 
     r = cyrusdb_store(seendb->db, uniqueid, strlen(uniqueid),
-		  data, datalen, &seendb->tid);
+                  data, datalen, &seendb->tid);
     switch (r) {
     case CYRUSDB_OK:
-	break;
+        break;
     case CYRUSDB_IOERROR:
-	r = IMAP_AGAIN;
-	break;
+        r = IMAP_AGAIN;
+        break;
     default:
-	syslog(LOG_ERR, "DBERROR: error updating database: %s", 
-	       cyrusdb_strerror(r));
-	r = IMAP_IOERROR;
-	break;
+        syslog(LOG_ERR, "DBERROR: error updating database: %s",
+               cyrusdb_strerror(r));
+        r = IMAP_IOERROR;
+        break;
     }
 
     free(data);
@@ -328,26 +330,26 @@ EXPORTED int seen_close(struct seen **seendbptr)
     if (!seendb) return 0;
 
     if (SEEN_DEBUG) {
-	syslog(LOG_DEBUG, "seen_db: seen_close(%s)", seendb->user);
+        syslog(LOG_DEBUG, "seen_db: seen_close(%s)", seendb->user);
     }
 
     if (seendb->tid) {
-	if (SEEN_DEBUG) {
-	    syslog(LOG_DEBUG, "seen_db: committing changes for %s", seendb->user);
-	}
-	r = cyrusdb_commit(seendb->db, seendb->tid);
-	if (r != CYRUSDB_OK) {
-	    syslog(LOG_ERR, "DBERROR: error committing seen txn; "
-		   "seen state lost: %s", cyrusdb_strerror(r));
-	}
-	seendb->tid = NULL;
+        if (SEEN_DEBUG) {
+            syslog(LOG_DEBUG, "seen_db: committing changes for %s", seendb->user);
+        }
+        r = cyrusdb_commit(seendb->db, seendb->tid);
+        if (r != CYRUSDB_OK) {
+            syslog(LOG_ERR, "DBERROR: error committing seen txn; "
+                   "seen state lost: %s", cyrusdb_strerror(r));
+        }
+        seendb->tid = NULL;
     }
 
     r = cyrusdb_close(seendb->db);
     if (r) {
-	syslog(LOG_ERR, "DBERROR: error closing: %s",
-	       cyrusdb_strerror(r));
-	r = IMAP_IOERROR;
+        syslog(LOG_ERR, "DBERROR: error closing: %s",
+               cyrusdb_strerror(r));
+        r = IMAP_IOERROR;
     }
     free(seendb->user);
     free(seendb);
@@ -360,8 +362,8 @@ EXPORTED int seen_close(struct seen **seendbptr)
 HIDDEN int seen_create_mailbox(const char *userid, struct mailbox *mailbox)
 {
     if (SEEN_DEBUG) {
-	syslog(LOG_DEBUG, "seen_db: seen_create_mailbox(%s, %s)", 
-	       userid, mailbox->uniqueid);
+        syslog(LOG_DEBUG, "seen_db: seen_create_mailbox(%s, %s)",
+               userid, mailbox->uniqueid);
     }
 
     /* noop */
@@ -375,17 +377,17 @@ EXPORTED int seen_delete_mailbox(const char *userid, struct mailbox *mailbox)
     const char *uniqueid = mailbox->uniqueid;
 
     if (SEEN_DEBUG) {
-	syslog(LOG_DEBUG, "seen_db: seen_delete_mailbox(%s, %s)", 
-	       userid, uniqueid);
+        syslog(LOG_DEBUG, "seen_db: seen_delete_mailbox(%s, %s)",
+               userid, uniqueid);
     }
 
     /* noop */
     if (!userid)
-	return 0;
+        return 0;
 
     r = seen_open(userid, SEEN_SILENT, &seendb);
     if (!r) r = cyrusdb_delete(seendb->db, uniqueid, strlen(uniqueid),
-			   &seendb->tid, 1);
+                           &seendb->tid, 1);
     seen_close(&seendb);
 
     return r;
@@ -394,8 +396,8 @@ EXPORTED int seen_delete_mailbox(const char *userid, struct mailbox *mailbox)
 int seen_create_user(const char *user)
 {
     if (SEEN_DEBUG) {
-	syslog(LOG_DEBUG, "seen_db: seen_create_user(%s)", 
-	       user);
+        syslog(LOG_DEBUG, "seen_db: seen_create_user(%s)",
+               user);
     }
 
     /* we'll be lazy here and create this when needed */
@@ -408,13 +410,13 @@ HIDDEN int seen_delete_user(const char *user)
     int r = 0;
 
     if (SEEN_DEBUG) {
-	syslog(LOG_DEBUG, "seen_db: seen_delete_user(%s)", 
-	       user);
+        syslog(LOG_DEBUG, "seen_db: seen_delete_user(%s)",
+               user);
     }
 
     if (unlink(fname) && errno != ENOENT) {
-	syslog(LOG_ERR, "error unlinking %s: %m", fname);
-	r = IMAP_IOERROR;
+        syslog(LOG_ERR, "error unlinking %s: %m", fname);
+        r = IMAP_IOERROR;
     }
 
     free(fname);
@@ -428,43 +430,43 @@ HIDDEN int seen_rename_user(const char *olduser, const char *newuser)
     int r = 0;
 
     if (SEEN_DEBUG) {
-	syslog(LOG_DEBUG, "seen_db: seen_rename_user(%s, %s)", 
-	       olduser, newuser);
+        syslog(LOG_DEBUG, "seen_db: seen_rename_user(%s, %s)",
+               olduser, newuser);
     }
 
     cyrus_mkdir(newfname, 0755);
     if (rename(oldfname, newfname) && errno != ENOENT) {
-	syslog(LOG_ERR, "error renaming %s to %s: %m", oldfname, newfname);
-	r = IMAP_IOERROR;
+        syslog(LOG_ERR, "error renaming %s to %s: %m", oldfname, newfname);
+        r = IMAP_IOERROR;
     }
 
     free(oldfname);
     free(newfname);
-    
+
     return r;
 }
 
 HIDDEN int seen_copy(const char *userid, struct mailbox *oldmailbox,
-	      struct mailbox *newmailbox)
+              struct mailbox *newmailbox)
 {
     if (SEEN_DEBUG) {
-	syslog(LOG_DEBUG, "seen_db: seen_copy %s (%s => %s)",
-	       userid ? userid : "", oldmailbox->uniqueid, newmailbox->uniqueid);
+        syslog(LOG_DEBUG, "seen_db: seen_copy %s (%s => %s)",
+               userid ? userid : "", oldmailbox->uniqueid, newmailbox->uniqueid);
     }
 
     if (userid && strcmp(oldmailbox->uniqueid, newmailbox->uniqueid)) {
-	int r;
-	struct seen *seendb = NULL;
-	struct seendata sd = SEENDATA_INITIALIZER;
+        int r;
+        struct seen *seendb = NULL;
+        struct seendata sd = SEENDATA_INITIALIZER;
 
-	r = seen_open(userid, SEEN_SILENT, &seendb);
-    
-	/* just be silent if it's missing */
-	if (!r) r = seen_lockread(seendb, oldmailbox->uniqueid, &sd);
-	if (!r) r = seen_write(seendb, newmailbox->uniqueid, &sd);
+        r = seen_open(userid, SEEN_SILENT, &seendb);
 
-	seen_close(&seendb);
-	seen_freedata(&sd);
+        /* just be silent if it's missing */
+        if (!r) r = seen_lockread(seendb, oldmailbox->uniqueid, &sd);
+        if (!r) r = seen_write(seendb, newmailbox->uniqueid, &sd);
+
+        seen_close(&seendb);
+        seen_freedata(&sd);
     }
 
     /* noop */
@@ -474,7 +476,7 @@ HIDDEN int seen_copy(const char *userid, struct mailbox *oldmailbox,
 EXPORTED int seen_done(void)
 {
     if (SEEN_DEBUG) {
-	syslog(LOG_DEBUG, "seen_db: seen_done()");
+        syslog(LOG_DEBUG, "seen_db: seen_done()");
     }
 
     return 0;
@@ -483,10 +485,10 @@ EXPORTED int seen_done(void)
 EXPORTED int seen_compare(struct seendata *a, struct seendata *b)
 {
     if (a->lastuid == b->lastuid &&
-	a->lastread == b->lastread &&
-	a->lastchange == b->lastchange &&
-	!strcmp(a->seenuids, b->seenuids))
-	return 1;
+        a->lastread == b->lastread &&
+        a->lastchange == b->lastchange &&
+        !strcmp(a->seenuids, b->seenuids))
+        return 1;
 
     return 0;
 }
@@ -495,8 +497,8 @@ EXPORTED int seen_compare(struct seendata *a, struct seendata *b)
  * last change times, and ensure that the database uses the newer of
  * the two */
 static int seen_merge_cb(void *rockp,
-			 const char *key, size_t keylen,
-			 const char *newdata, size_t newlen) 
+                         const char *key, size_t keylen,
+                         const char *newdata, size_t newlen)
 {
     int r = 0;
     struct seen *seendb = (struct seen *)rockp;
@@ -507,16 +509,16 @@ static int seen_merge_cb(void *rockp,
     parse_data(newdata, newlen, &newsd);
 
     if (seen_lockread(seendb, uniqueid, &oldsd)) {
-	dirty = 1; /* no record */
+        dirty = 1; /* no record */
     }
     else {
-	if (newsd.lastuid > oldsd.lastuid) dirty = 1;
-	if (newsd.lastread > oldsd.lastread) dirty = 1;
+        if (newsd.lastuid > oldsd.lastuid) dirty = 1;
+        if (newsd.lastread > oldsd.lastread) dirty = 1;
     }
-    
+
     if (dirty) {
-	/* write back data from new entry */
-	r = seen_write(seendb, uniqueid, &newsd);
+        /* write back data from new entry */
+        r = seen_write(seendb, uniqueid, &newsd);
     }
 
     free(uniqueid);
