@@ -48,7 +48,6 @@
 
 #include <string.h>
 #include <sys/types.h>
-#include "util.h"
 
 typedef struct
 {
@@ -80,6 +79,10 @@ const char *strarray_safenth(const strarray_t *sa, int idx);
 void strarray_truncate(strarray_t *sa, int newlen);
 strarray_t *strarray_dup(const strarray_t *);
 void strarray_cat(strarray_t *dest, const strarray_t *src);
+void strarray_swap(strarray_t *, int, int);
+void strarray_addfirst(strarray_t *, const char *);
+void strarray_addfirst_case(strarray_t *, const char *);
+void strarray_subtract_complement(strarray_t *sa, const strarray_t *sb);
 
 #define strarray_shift(sa)          strarray_remove((sa), 0)
 #define strarray_unshift(sa, s)     strarray_insert((sa), 0, (s))
@@ -95,9 +98,13 @@ strarray_t *strarray_splitm(char *buf, const char *sep, int flags);
 strarray_t *strarray_split(const char *buf, const char *sep, int flags);
 strarray_t *strarray_nsplit(const char *buf, size_t len, const char *sep, int flags);
 
-void strarray_sort(strarray_t *, compar_t *);
+/* strarray_cmp_fn_t is same sig as qsort's compar argument */
+typedef int strarray_cmp_fn_t(const void *, const void *);
+void strarray_sort(strarray_t *, strarray_cmp_fn_t *);
+
 void strarray_uniq(strarray_t *);
 
+char **strarray_safetakevf(strarray_t *sa);
 char **strarray_takevf(strarray_t *sa);
 
 int strarray_find(const strarray_t *sa, const char *match,
@@ -105,6 +112,9 @@ int strarray_find(const strarray_t *sa, const char *match,
 int strarray_find_case(const strarray_t *sa, const char *match,
                        int starting);
 
+int strarray_intersect(const strarray_t *sa, const strarray_t *b);
+int strarray_intersect_case(const strarray_t *sa, const strarray_t *b);
+
 int strarray_size(const strarray_t *sa);
 
 int strarray_cmp(const strarray_t *a, const strarray_t *b);
