@@ -61,70 +61,71 @@ static int contains_8bit(const char *msg);
 static int global_outgoing_count = 0;
 
 char* notify_mailto(const char *class,
-		    const char *priority __attribute__((unused)),
-		    const char *user __attribute__((unused)),
-		    const char *mailbox __attribute__((unused)),
-		    int nopt, char **options,
-		    const char *message)
+                    const char *priority __attribute__((unused)),
+                    const char *user __attribute__((unused)),
+                    const char *mailbox __attribute__((unused)),
+                    int nopt, char **options,
+                    const char *message,
+                    const char *fname __attribute__((unused)))
 {
     FILE *sm;
     const char *smbuf[7];
     char outmsgid[256];
     int sm_stat;
     time_t t;
-    char datestr[RFC822_DATETIME_MAX+1];
+    char datestr[RFC5322_DATETIME_MAX+1];
     pid_t sm_pid;
     int fds[2];
 
     /* XXX check/parse options (mailto URI) */
     if (nopt < 1)
-	return strdup("NO mailto URI not specified");
+        return strdup("NO mailto URI not specified");
 
     smbuf[0] = "sendmail";
-    smbuf[1] = "-i";		/* ignore dots */
+    smbuf[1] = "-i";            /* ignore dots */
     smbuf[2] = "-f";
-    smbuf[3] = "<>";		/* XXX do we want a return-path? */
+    smbuf[3] = "<>";            /* XXX do we want a return-path? */
     smbuf[4] = "--";
     smbuf[5] = options[0];
     smbuf[6] = NULL;
 
     if (pipe(fds))
-	return strdup("NO mailto could not open pipe");
+        return strdup("NO mailto could not open pipe");
 
     if ((sm_pid = fork()) == 0) {
-	/* i'm the child! run sendmail! */
-	close(fds[1]);
-	/* make the pipe be stdin */
-	dup2(fds[0], 0);
-	execv(config_getstring(IMAPOPT_SENDMAIL), (char **) smbuf);
+        /* i'm the child! run sendmail! */
+        close(fds[1]);
+        /* make the pipe be stdin */
+        dup2(fds[0], STDIN_FILENO);
+        execv(config_getstring(IMAPOPT_SENDMAIL), (char **) smbuf);
 
-	/* if we're here we suck */
-	return strdup("NO mailto couldn't exec");
+        /* if we're here we suck */
+        return strdup("NO mailto couldn't exec");
     }
     /* i'm the parent */
     close(fds[0]);
     sm = fdopen(fds[1], "w");
 
     if (!sm)
-	return strdup("NO mailto could not spawn sendmail process");
+        return strdup("NO mailto could not spawn sendmail process");
 
     t = time(NULL);
-    snprintf(outmsgid, sizeof(outmsgid), "<cmu-sieve-%d-%lu-%d@%s>", 
-	     (int) sm_pid, t, global_outgoing_count++, config_servername);
-    
+    snprintf(outmsgid, sizeof(outmsgid), "<cmu-sieve-%d-%lu-%d@%s>",
+             (int) sm_pid, t, global_outgoing_count++, config_servername);
+
     fprintf(sm, "Message-ID: %s\r\n", outmsgid);
 
-    time_to_rfc822(t, datestr, sizeof(datestr));
+    time_to_rfc5322(t, datestr, sizeof(datestr));
     fprintf(sm, "Date: %s\r\n", datestr);
-    
+
     fprintf(sm, "X-Sieve: %s\r\n", SIEVE_VERSION);
     fprintf(sm, "From: Mail Sieve Subsystem <%s>\r\n", config_getstring(IMAPOPT_POSTMASTER));
     fprintf(sm, "To: <%s>\r\n", options[0]);
     fprintf(sm, "Subject: [%s] New mail notification\r\n", class);
     if (contains_8bit(message)) {
-	fprintf(sm, "MIME-Version: 1.0\r\n");
-	fprintf(sm, "Content-Type: text/plain; charset=UTF-8\r\n");
-	fprintf(sm, "Content-Transfer-Encoding: 8BIT\r\n");
+        fprintf(sm, "MIME-Version: 1.0\r\n");
+        fprintf(sm, "Content-Type: text/plain; charset=UTF-8\r\n");
+        fprintf(sm, "Content-Transfer-Encoding: 8BIT\r\n");
     }
     fprintf(sm, "\r\n");
 
@@ -145,15 +146,15 @@ static int contains_8bit(const char * msg)
     int result = 0;
 
     if (msg) {
-	const unsigned char *s = (const unsigned char *)msg;
-	
-	while (*s) {
-	    if (0 != (*s & 0x80)) {
-		result = 1;
-		break ;
-	    }
-	    s++;
-	}
+        const unsigned char *s = (const unsigned char *)msg;
+
+        while (*s) {
+            if (0 != (*s & 0x80)) {
+                result = 1;
+                break ;
+            }
+            s++;
+        }
     }
     return result;
 }
